
run_SAMC <- function(data, cc.var, exposure.var, group.var, out.dir, op=NULL) {

  # Make sure out.dir ends with a folder delimiter
  out.dir <- checkForSep(out.dir)

  ###############################################################################
  ###############################################################################
  ###############################################################################
  ###############################################################################
  ###############################################################################

  op <- default.list(op, 
         c("alpha_min", "alpha_max", "beta_min", "beta_max",
           "sigma2_beta", "sigma2_alpha", "sigma2_psi",
           "psi_k", "psi_theta", "psi_min", "psi_max",
           "w_m", "update_prop", "n_iter", "n_sep_out",
           "random_seed", "cluster_size", "k_max",
           "method_psi", "tao_min", "tao_max", "sigma2_tao",
           "split_option", "shape_split_a", "shape_split_b",
           "shape_merge_a", "shape_merge_b", "update_prop_w",
           "out.string",
           "method_alpha", "method_beta", "method_tao",
           "alpha_mu", "alpha_sigma2", "beta_mu", "beta_sigma2",
           "tao_mu", "tao_sigma2",
           "method",
           "delete", "num_iter", "gain_factor_t0", "psi_vec"),
          list(-4, 4, -4, 4,
               0.005, 0.005, 0.02,
               4.0, 0.2, 0.0, 1.2,
               50, 1.0, 200000, 1,
               12345, 0.0, 2, 
               0, -4, 4, 0.005,
               0, 2, 2, 
               2, 2, 1,
               "",
               0, 0, 0,
               0, 4, 0, 4,
               0, 4,
               0,
               1, 10000000, 50000, 0.1*c(0:12)
               )
          )
  op$num_of_cluster <- op$k_max
  op$clusters <- op$k_max

  if (!is.data.frame(data)) stop("ERROR: data must be a data frame")
  if (is.null(cc.var)) stop("ERROR: cc.var must be specified")
  if (is.null(exposure.var)) stop("ERROR: exposure.var must be specified")
  if (is.null(group.var)) stop("ERROR: group.var must be specified")
  if (is.null(out.dir)) stop("ERROR: out.dir must be specified")

  method <- op$method
  if (!(method %in% 0:1)) stop("ERROR: method must be 0 or 1")
  op$method_alpha <- method
  op$method_beta  <- method
  op$method_tao   <- method

  covars     <- op[["covars", exact=TRUE]]
  ncovars    <- length(covars)
  covarsFlag <- !is.null(covars)
  out.dir    <- checkForSep(out.dir)
  sim.mat    <- op[["sim.mat", exact=TRUE]]
  simFlag    <- !is.null(sim.mat)

  # Check the length of the tao parameters
  if (ncovars) {
    op$tao_min <- paste(rep(op$tao_min, ncovars)[1:ncovars], collapse=",", sep="")
    op$tao_max <- paste(rep(op$tao_max, ncovars)[1:ncovars], collapse=",", sep="")
    op$tao_mu  <- paste(rep(op$tao_mu, ncovars)[1:ncovars], collapse=",", sep="")
    op$tao_sigma2 <- paste(rep(op$tao_sigma2, ncovars)[1:ncovars], collapse=",", sep="")
  }

  # Check variable names
  vlist <- list(response.var=cc.var, snp.var=exposure.var, main.vars=covars,
               group.var=group.var)
  vars <- getAllVars(vlist, names=names(vlist))
  temp <- !(vars %in% colnames(data))
  if (any(temp)) {
    print(vars[temp])
    stop("The above variables were not found in the input data")
  }

  # Check the variables
  if (!all(data[, cc.var] %in% 0:1)) stop("ERROR: cc.var must be coded as 0-1")
  if (!is.numeric(data[, exposure.var])) stop("ERROR: exposure.var must be numeric")

  # Check the group variable 
  temp   <- data[, group.var]
  ngroup <- length(unique(temp))
  if (!all(temp %in% 1:ngroup)) stop("ERROR: group.var must be coded as 1, 2, 3, ...")
  data[, group.var] <- as.numeric(temp) - 1

  # Check the similarity matrix 
  if (simFlag) {
    sim.mat <- as.matrix(sim.mat)
    if (nrow(sim.mat) != ncol(sim.mat)) stop("ERROR: sim.mat must be a square matrix")
    if (any(is.na(sim.mat))) stop("ERROR: sim.mat cannot contain missing values")
    if (max(abs(sim.mat - t(sim.mat))) > 1e-5) stop("ERROR: sim.mat must be symmetric")
    if (ngroup != nrow(sim.mat)) stop("ERROR: dimension of sim.mat is not correct")
    if (any(diag(sim.mat) != 0)) stop("ERROR: diagonal of sim.mat should be 0")
  } else {
    sim.mat <- matrix(data=0, nrow=ngroup, ncol=ngroup)
  }

  # Check for missing values
  nsub <- nrow(data)
  temp <- getFormulas(vlist)
  miss <- c(NA, NaN, Inf, -Inf)
  if (length(temp)) data <- applyFormulas(data, temp, remove=miss)
  data <- removeMiss.vars(data, vars=vars, miss=miss)
  if (nsub != nrow(data)) stop("ERROR: data must not contain missing values")

  # Get the design matrix
  if (covarsFlag) {
    design <- removeOrKeepCols(data, covars)
  } else {
    design <- NULL
  }

  # Create file names
  str <- op$out.string
  data.file <- paste(out.dir, str, "_TEMP_SAMC_data.txt", sep="")
  sim.file  <- paste(out.dir, str, "_TEMP_SAMC_sim.txt", sep="")

  # Create the parameter file
  names <- c("alpha_min", "alpha_max", "beta_min", "beta_max",
           "sigma2_beta", "sigma2_alpha", "sigma2_psi",
           "psi_k", "psi_theta", "psi_min", "psi_max",
           "w_m", "update_prop", "n_iter", "n_sep_out",
           "random_seed", "cluster_size",
           "method_psi", "tao_min", "tao_max", "sigma2_tao",
           "split_option", "shape_split_a", "shape_split_b",
           "shape_merge_a", "shape_merge_b", "update_prop_w",
           "method_alpha", "method_beta", "method_tao",
           "alpha_mu", "alpha_sigma2", "beta_mu", "beta_sigma2",
           "tao_mu", "tao_sigma2", "psi_file")
  out <- c("out_k", "out_psi", "out_alpha", "out_beta", "out_alloc_z",
           "out_dev_i", "out_tao")
  n <- length(names) + length(out) + 2
  x <- character(n)
  x[1]  <- "NAME,VALUE"
  x[2]  <- paste("data,", data.file, sep="")
  x[3]  <- paste("sim_matrix,", sim.file, sep="")
  index <- 4
  n.out <- length(out)
  out.files <- character(n.out)
  for (i in 1:n.out) {
    nn   <- out[i]
    temp <- paste(out.dir, str, nn, "_SAMC.txt", sep="")
    out.files[i] <- temp
    x[index] <- paste(nn, ",", temp, sep="")
    index <- index + 1
  }
  for (nn in names) {
    temp <- op[[nn, exact=TRUE]]
    x[index] <- paste(nn, ",", temp, sep="")
    index <- index + 1
  }

  # Write the similarity matrix
  write.table(sim.mat, file=sim.file, col.names=FALSE, sep=" ", row.names=FALSE, quote=FALSE)

  # Write out the data file
  data <- cbind(data[, cc.var], data[, exposure.var], data[, group.var], design)
  write.table(data, file=data.file, col.names=FALSE, sep=" ", row.names=FALSE, quote=FALSE)
  argc <- 2
  ret  <- 1
  if (op$random_seed > 0) set.seed(op$random_seed)
  

  ###############################################################################
  ###############################################################################
  ###############################################################################
  ###############################################################################
  ###############################################################################

  cfg  <- x
  rm(x)
  
  # Save the config options
  cfg0 <- cfg
  delete         <- op$delete
  clusters       <- op$clusters
  psi_vec        <- op$psi_vec
  gain_factor_t0 <- op$gain_factor_t0
  num_iter       <- op$num_iter

  # Loop over each value of clusters
  for (clus in clusters) {
    cfg <- cfg0

    # Set the number of clusters
    temp <- paste("k_max,", clus, sep="")
    cfg  <- c(cfg, temp)
    temp <- paste("num_of_cluster,", clus,  sep="")
    cfg  <- c(cfg, temp)

    # Create a temporary config file 
    temp.config <- paste(out.dir, "temp_config_", str, "_", clus, sep="")

    # Write out the config file
    write(cfg, file=temp.config, ncolumns=1)

    # Define parameters for the normC_SAMC function
    num_grid    <- length(psi_vec)
    theta_start <- rep(0, num_grid)
    hist_mat    <- double(num_grid*3)
    narg        <- 1

    # Call the function
    ret <- .C("normC_SAMC", as.integer(narg), temp.config, as.double(psi_vec), as.integer(clus), as.integer(num_grid), 
    as.integer(num_iter), as.double(gain_factor_t0), as.double(theta_start), hist_mat=hist_mat)

    # Write out the output
    hist_mat    <- matrix(ret$hist_mat, byrow=TRUE, ncol=3)
    psi.mat     <- matrix(0, ncol=3, nrow=num_grid)
    psi.mat[,1] <- psi_vec
    psi.mat[,2] <- rep(1/num_grid, num_grid)
    psi.mat[,3] <- hist_mat[,2]
    out.1       <- paste(out.dir, "psi_grid", str, "_", clus, ".txt", sep="")
    #out.2       <- paste(out.dir, "hist", str, "_", clus, sep="")   
   
    write.table(psi.mat, file=out.1, quote=FALSE, row.names=FALSE, col.names=FALSE)

    #write.table(hist_mat, file=out.2, quote=FALSE, row.names=FALSE, col.names=FALSE)

    # Delete temporary files
    if (delete) file.remove(c(temp.config))
  }

  if (delete) file.remove(c(data.file, sim.file))


  NULL
}


